@@ -7,20 +7,72 @@ Augmentation parameters can be `Constant`, or random with `Normal` or `Uniform`
 Random parameters will be sampled from a given distribution anew for each augmentation.
 
 `aug = Gain(gain=Normal(mean=1, std=0.5)); augmented_signal = aug(signal)`
+
+Can be configured using a yaml file:
+```yaml
+Gain:  # Name of the augmentation class
+  gain:  # arg for the augmentation class
+    Uniform:  # Param type
+       lower: 0.5  # param args
+       upper: 2  # param args
+
+MaskNoise:
+  std:
+    Normal:
+      mean: 0
+      std: 0.05
+  mean:
+    Constant:
+       value: 0
+
+NotchFilter:
+  freq:  # Param-type arg
+    Uniform:
+      lower: 100
+      upper: 600
+  Q: 30  # standard arg
+  samplerate_Hz: 10_000  # standard arg
+```
+Caution: You need to add a suffix starting with '-' (like "MaskNoise-1") to the class name if you want to use a class multiple times
+
+`augs = Augmentations.from_yaml(filename)`
+
+
 """
 import numpy as np
 from typing import List, Optional, Callable
 import scipy.signal
+import yaml
+from dataclasses import dataclass
+from typing import Dict
+import logging
 
 
+aug_dict = dict()
+params_dict = dict()
+
+
+def _register_augmentation(func):
+    """Adds func to model_dict Dict[augname: augfunc]. For selecting augs by string."""
+    aug_dict[func.__name__] = func
+    return func
+
+
+def _register_param(func):
+    """Adds func to model_dict Dict[paramname: paramfunc]. For selecting params by string."""
+    params_dict[func.__name__] = func
+    return func
+
+
+@dataclass
 class Param():
     """Base class for all parameters.
 
     Parameters are callables that return parameter values.
     """
-    pass
 
 
+@_register_param
 class Constant(Param):
     """Constant parameter."""
 
@@ -34,7 +86,14 @@ class Constant(Param):
     def __call__(self, shape=1) -> np.ndarray:
         return self.value * np.ones(shape)
 
+    def __str__(self):
+        return f"{self.__class__.__name__}(value={self.value})"
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}(value={self.value})"
+
+
+@_register_param
 class Normal(Param):
     """Normally distributed parameter."""
 
@@ -50,7 +109,14 @@ class Normal(Param):
     def __call__(self, shape=1) -> np.ndarray:
         return np.random.normal(self.mean, self.std, size=shape)
 
+    def __str__(self):
+        return f"{self.__class__.__name__}(mean={self.mean}, std={self.std})"
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}(mean={self.mean}, std={self.std})"
+
+
+@_register_param
 class Uniform(Param):
     """Uniformly distributed parameter."""
     def __init__(self, lower: float = -1.0, upper: float = 1.0):
@@ -65,12 +131,23 @@ class Uniform(Param):
     def __call__(self, shape=1) -> np.ndarray:
         return np.random.uniform(self.lower, self.upper, size=shape)
 
+    def __str__(self):
+        return f"{self.__class__.__name__}(lower={self.lower}, upper={self.upper})"
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}(lower={self.lower}, upper={self.upper})"
+
+
+@dataclass
 class Augmentation(Callable):
     """Base class for all augmentations.
 
     Augmentations are callables that return the augmented input.
-    Can optionally pass through a second input."""
+    Can optionally pass through a second input.
+
+    batch_x is expected to be [batch, time, channel].
+    _apply works on [time, channel] inputs
+    """
     def __call__(self, batch_x, batch_y=None):
         this_batch_x = batch_x.copy()
         for batch in range(batch_x.shape[0]):
@@ -81,6 +158,7 @@ class Augmentation(Callable):
             return this_batch_x, batch_y
 
 
+@_register_augmentation
 class Gain(Augmentation):
     """Multiply signal with gain factor."""
 
@@ -95,7 +173,11 @@ class Gain(Augmentation):
         x *= self.gain()
         return x
 
+    # def __repr__(self):
+    #     return f"{self.__class__.__name__}(gain={self.gain.__repr__()})"
 
+
+@_register_augmentation
 class Offset(Augmentation):
     """Add horizontal offset."""
 
@@ -111,6 +193,7 @@ class Offset(Augmentation):
         return x
 
 
+@_register_augmentation
 class HorizontalFlip(Augmentation):
     """Horizontally flip signal."""
 
@@ -127,6 +210,7 @@ class HorizontalFlip(Augmentation):
         return x
 
 
+@_register_augmentation
 class Upsampling(Augmentation):
     """Upsample signal."""
 
@@ -149,6 +233,7 @@ class Upsampling(Augmentation):
         return x
 
 
+@_register_augmentation
 class MaskNoise(Augmentation):
     """Add noise or replace signal by noise for the full duration or a part of it."""
 
@@ -176,14 +261,18 @@ class MaskNoise(Augmentation):
         else:
             duration = int(self.duration())
             mask_start = np.random.randint(low=0, high=len_x - duration)
-        noise = np.random.randn(duration) * self.std() + self.mean(shape=(1, *x.shape[1:]))
+        noise = np.random.randn(duration, *x.shape[1:]) * self.std() + self.mean()
         if self.add:
-            x[mask_start:mask_start + duration] += noise
+            x[mask_start:mask_start + duration, :] += noise
         else:
-            x[mask_start:mask_start + duration] = noise
+            x[mask_start:mask_start + duration, :] = noise
         return x
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}(std={self.mean}, mean={self.mean})"
 
+
+@_register_augmentation
 class MaskMean(Augmentation):
     """Replaces stretch of `duration` samples with mean over that stretch."""
 
@@ -196,11 +285,13 @@ class MaskMean(Augmentation):
 
     def _apply(self, x):
         len_x = x.shape[0]
-        mask_start = np.random.randint(low=0, high=len_x - self.duration)
-        x[mask_start:mask_start + self.duration, :] = np.mean(x[mask_start:mask_start + self.duration], axis=0)
+        duration = int(self.duration())
+        mask_start = np.random.randint(low=0, high=len_x - duration)
+        x[mask_start:mask_start + duration, :] = np.mean(x[mask_start:mask_start + duration], axis=0)
         return x
 
 
+@_register_augmentation
 class CircShift(Augmentation):
     """Circularly shift input along the first axis."""
 
@@ -216,13 +307,32 @@ class CircShift(Augmentation):
         return x
 
 
+@_register_augmentation
+class NotchFilter(Augmentation):
+    """Notch filter."""
+
+    def __init__(self, freq: Param, Q: float = 30, samplerate_Hz: float = 10_000):
+        """
+        Args:
+            freq (Param): freq to remove, in Hz.
+        """
+        self.freq = freq
+        self.Q = Q
+        self.samplerate_Hz = samplerate_Hz
+
+    def _apply(self, x):
+        b, a = scipy.signal.iirnotch(self.freq(), self.Q, self.samplerate_Hz)
+        x = scipy.signal.filtfilt(b, a, x, axis=0)
+        return x
+
+
 class Augmentations():
     """Bundles several augmentations."""
 
     def __init__(self, augmentations: List[Augmentation]):
         """
         Args:
-            augmentations (List[Augmentation]): List of augmentations.
+            augmentations (List[Augmentation]): List of Augmentation instances.
         """
         self.augmentations = augmentations
 
@@ -233,3 +343,34 @@ class Augmentations():
             return batch_x
         else:
             return batch_x, batch_y
+
+    def __len__(self):
+        return len(self.augmentations)
+
+    @classmethod
+    def from_yaml(cls, filename: str):
+        aug_spec = yaml.safe_load(open(filename, 'r'))
+        return cls.from_dict(aug_spec)
+
+    @classmethod
+    def from_dict(cls, aug_spec: Dict):
+        augs = []
+        for name, args in aug_spec.items():
+            name = name.split('-', 1)[0]  # split off "-WHATEVER" suffix
+            params = dict()
+            for a_name, a_arg in args.items():
+                if isinstance(a_arg, dict): # Param-type arg
+                    p_name = list(a_arg.keys())[0]
+                    p_args = a_arg[p_name]
+                    # if no args for Params are provided, use defaults
+                    if p_args is None:
+                        p_args = {}
+                    params[a_name] = params_dict[p_name](**p_args)
+                else:  # standard arg
+                    params[a_name] = a_arg
+
+            print(params)
+            aug = aug_dict[name](**params)
+            print(aug)
+            augs.append(aug)
+        return cls(augs)
