@@ -7,11 +7,7 @@ import logging
 import flammkuchen as fl
 import numpy as np
 
-try:
-    from tensorflow.python.keras.utils import tf_utils
-except ImportError:
-    from tensorflow.keras.utils import tf_utils
-
+from tensorflow.python.keras.utils import tf_utils
 from tensorflow.keras.callbacks import Callback, EarlyStopping, ModelCheckpoint, ReduceLROnPlateau, TensorBoard
 from tensorflow import keras
 import keras_tuner as kt
@@ -21,12 +17,6 @@ import sys
 from typing import List, Optional, Tuple, Dict, Any
 from . import data, models, utils, predict, io, evaluate, tracking, data_hash  #, timeseries
 
-try:  # disabling eager execution speeds up everything
-    from tensorflow.python.framework.ops import disable_eager_execution
-    disable_eager_execution()
-except Exception as e:
-    logging.exception(e)
-
 try:  # fixes cuDNN error when using LSTM layer
     import tensorflow as tf
     physical_devices = tf.config.list_physical_devices('GPU')
